@@ -569,7 +569,7 @@ def render_rubric_table(taskid, table, blank=False):
     string. Returns an HTML string that builds a div with class
     "rubric_table".
 
-    blank can be set to True (instead of the default False) to make
+    `blank` can be set to True (instead of the default False) to make
     rubric details more visible in blank rubrics where explanations
     haven't been generated yet, but it means that explanations will not
     be rendered in the output (warnings and notes still will be). Setting
