@@ -21,6 +21,7 @@ import flask, redis, bs4, werkzeug
 
 import potluck.time_utils, potluck.html_tools, potluck.render
 
+
 # Python 2/3 dual compatibility
 if sys.version_info[0] < 3:
     reload(sys) # noqa F821
@@ -33,6 +34,19 @@ else:
     IOError_or_FileNotFoundError = FileNotFoundError
     OSError_or_FileNotFoundError = FileNotFoundError
 
+
+# Look for safe_join both in flask and in werkzeug...
+if hasattr(flask, "safe_join"):
+    safe_join = flask.safe_join
+elif hasattr(werkzeug.utils, "safe_join"):
+    safe_join = werkzeug.utils.safe_join
+else:
+    print(
+        "Warning: safe_join was not found in either flask OR"
+        " werkzeug.utils; using an unsafe function instead."
+    )
+    safe_join = lambda *args: os.path.join(*args)
+
 #-----------#
 # Constants #
 #-----------#
@@ -111,7 +125,7 @@ def logs_folder(course, semester, username):
     """
     The logs folder for a class/semester/user.
     """
-    return werkzeug.utils.safe_join(
+    return safe_join(
         evaluation_directory(course, semester),
         "logs",
         username
@@ -122,7 +136,7 @@ def reports_folder(course, semester, username):
     """
     The reports folder for a class/semester/user.
     """
-    return werkzeug.utils.safe_join(
+    return safe_join(
         evaluation_directory(course, semester),
         "reports",
         username
@@ -911,14 +925,14 @@ def put_inflight(course, semester, username, phase, psid, taskid):
             log_folder = logs_folder(course, semester, username)
             ensure_directory(log_folder)
             logfile = unused_filename(
-                werkzeug.utils.safe_join(log_folder, istring + ".log")
+                safe_join(log_folder, istring + ".log")
             )
 
             # Select an unused report filename
             report_folder = reports_folder(course, semester, username)
             ensure_directory(report_folder)
             reportfile = unused_filename(
-                werkzeug.utils.safe_join(report_folder, istring + ".json")
+                safe_join(report_folder, istring + ".json")
             )
 
             # Gather the info into a tuple
@@ -1151,6 +1165,9 @@ def fetch_evaluation(course, semester, username, phase, psid, taskid):
     - "override": A numerical score that overrides the automatic
         evaluation. Will be an empty string if there is no override to
         apply.
+    - "timeliness": A numerical score for timeliness points to override
+        the automatic value. Will be an empty string if there is no
+        override, which should always be the case for non-initial phases.
 
     Returns None instead of a dictionary if there is no information for
     that user/pset/task yet, or if an error is encountered while trying
@@ -1171,7 +1188,8 @@ def fetch_evaluation(course, semester, username, phase, psid, taskid):
             key,
             "updated_at",
             "notes",
-            "override"
+            "override",
+            "timeliness"
         )
     except Exception:
         flask.flash("Error fetching evaluation info.")
@@ -1186,7 +1204,11 @@ def fetch_evaluation(course, semester, username, phase, psid, taskid):
         return None
 
     # Some kind of non-exception error during access, or key is missing
-    if response is None or len(response) != 3 or response[0] is None:
+    if (
+        response is None
+     or len(response) != 4
+     or response[0] is None
+    ):
         return None
 
     try:
@@ -1194,6 +1216,11 @@ def fetch_evaluation(course, semester, username, phase, psid, taskid):
     except ValueError:
         override = response[2]
 
+    try:
+        timeliness = float(response[3])
+    except ValueError:
+        timeliness = response[3]
+
     result = {
         "phase": phase,
         "psid": psid,
@@ -1201,6 +1228,7 @@ def fetch_evaluation(course, semester, username, phase, psid, taskid):
         "updated_at": response[0],
         "notes": response[1],
         "override": override,
+        "timeliness": timeliness,
     }
 
     return result
@@ -1214,7 +1242,8 @@ def set_evaluation(
     psid,
     taskid,
     notes,
-    override=""
+    override="",
+    timeliness=""
 ):
     """
     Updates the custom evaluation info for a particular task submitted by
@@ -1233,6 +1262,10 @@ def set_evaluation(
     a floating-point number or integer between 0 and 100; it will be
     stored as a float if convertible.
 
+    The timeliness argument works like the override argument, but
+    overrides the timeliness score. It should only be set for the
+    'initial' phase.
+
     Returns True if it succeeds or False if it encounters some sort of
     error.
     """
@@ -1250,33 +1283,60 @@ def set_evaluation(
     timestring = potluck.time_utils.timestring()
 
     # Convert to a number if we can
+    score_basis = _CONFIG.get("SCORE_BASIS", 100)
     if override != "":
         try:
             override = float(override)
-            if 0 < override < 1:
+            if 0 < override < 1 and score_basis >= 10:
                 flask.flash(
                     (
                         "Warning: you entered '{}' as the grade"
                         " override, but scores should be specified out"
-                        " of 100, not out of 1! The override has been"
+                        " of {}, not out of 1! The override has been"
                         " set as-given but you may want to update it."
-                    ).format(override)
+                    ).format(override, score_basis)
                 )
         except Exception:
             flask.flash(
                 (
                     "Warning: you entered '{}' as the grade override,"
                     " but grade overrides should be numbers between 0"
-                    " and 100. The override has been set as-given, but"
+                    " and {}. The override has been set as-given, but"
                     " you may want to update it."
-                ).format(override)
+                ).format(override, score_basis)
+            )
+
+    # Convert to a number if we can
+    timeliness_basis = _CONFIG.get("TIMELINESS_POINTS", 10)
+    if timeliness != "":
+        try:
+            timeliness = float(timeliness)
+            if 0 < timeliness < 1 and timeliness_basis >= 5:
+                flask.flash(
+                    (
+                        "Warning: you entered '{}' as the timeliness"
+                        " override, but timeliness scores should be"
+                        " specified out of {}, not out of 1! The"
+                        " override has been set as-given but you may"
+                        " want to update it."
+                    ).format(timeliness, timeliness_basis)
+                )
+        except Exception:
+            flask.flash(
+                (
+                    "Warning: you entered '{}' as the grade override,"
+                    " but timeliness overrides should be numbers"
+                    " between 0 and {}. The override has been set"
+                    " as-given, but you may want to update it."
+                ).format(override, timeliness_basis)
             )
 
     # Here's the info we store
     info = {
         "updated_at": timestring,
         "notes": notes,
-        "override": override
+        "override": override,
+        "timeliness": timeliness,
     }
 
     try:
